/*********************************************************************

          ps.c: PolhemusSensor device driver
                    for ISOTRAK(II) (made from FASTRAK ver.)
                                        Kenji with Tail

*********************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include "ps.h"

/* ChangeUnit_ps() changes unit inchi/meter
 *    Returns False (zero) if buffer is full
 *    Returns True (non-zero) if successful
 *    give unit:  inchi: INCHI_PS/ meter: METER_PS.
 */
#define INCHI_PS 0
#define METER_PS 1
int ChangeUnit_ps(int unit);

/* ChangeFormat_ps() changes format ascii/binary
 *    Returns False (zero) if buffer is full
 *    Returns True (non-zero) if successful
 *    give form:  ascii: ASCII_PS/ binary: BINARY_PS.
 */
#define ASCII_PS 0
#define BINARY_PS 1
int ChangeFormat_ps(int form);

#define CTL_COM(n) ((n) - 0x40)
static	int	port_ref = -1;

int Init_ps(char *port, long int baud)
{
	int  result = 0;
	char  cmd[100];

	if (port_ref != -1){
		fprintf(stderr, "serial port has been opened");
		return result;
	}

	if ((port_ref = open(port, O_RDWR)) < 0){
		port_ref = -1;
		return result;
	}

	sprintf(cmd, "stty %ld -icanon raw -echo < %s", baud, port);
	system(cmd);

	fprintf(stderr, "Init_ps: ");
	Reset_ps();
	ChangeFormat_ps(ASCII_PS); /* please use ONLY ascii mode!!! */
	ChangeUnit_ps(METER_PS); /* default is METER, you can change. */
	result = 1;

	return result;
}

int Close_ps(void)
{
	int result = 0;

	if (port_ref != -1){
		close (port_ref);
		port_ref = -1;
		result = 1;
	} else {
		fprintf(stderr, "ps has not been opened\n");
		result = 0;
	}

	return result;
}

#define WTIMErps 5
int Reset_ps(void)
{
	char c = CTL_COM('Y');
	int result, i;

	result = (write (port_ref, &c, 1) == 1);
	fprintf(stderr, "Reset_ps: now resetting ps, please wait %d sec",
		WTIMErps);
	for (i = 0; i < WTIMErps; i ++){
		sleep(1);
		fprintf(stderr, ".");
	}
	fprintf(stderr, "done.\n");

	return result;
}

int Request_ps(PosOri_ps p1, PosOri_ps p2)
{
	char line[100], rcv[3], dmy;
	int i, btn = 0;

	write (port_ref, "P", 1);

	for (i = 0; read (port_ref, &line[i], 1) && line[i] >= ' '; i++);

	read (port_ref, &dmy, 1);

	line[i] = '\0';

	if (strstr(line, "ERR") != NULL) {
		return -1;
	}

	if (line[1] == '1'){
		sscanf(line, "%s %f %f %f %f %f %f",
		       rcv, p1, p1+1, p1+2, p1+3, p1+4, p1+5);
	} else if (line[1] == '2'){
		sscanf(line, "%s %f %f %f %f %f %f",
		       rcv, p2, p2+1, p2+2, p2+3, p2+4, p2+5);
	} else {
		return -1;
	}

	return (int) rcv[1];
}


int ChangeUnit_ps(int unit)
{
	switch (unit){
	    case INCHI_PS:
		return (write(port_ref, "U", 1) == 1);
	    case METER_PS:
		return (write(port_ref, "u", 1) == 1);
	    default:
		fprintf(stderr,
			"ChangeUnit_ps: please give INCHI_PS/METER_PS.\n");
		return -1;
	}
}

int ChangeFormat_ps(int form)
{
	switch (form){
	    case ASCII_PS:
		return (write(port_ref, "F", 1) == 1);
	    case BINARY_PS:
		return (write(port_ref, "f", 1) == 1);
	    default:
		fprintf(stderr,
			"ChangeFormat_ps: please give ASCII_PS/BINARY_PS.\n");
		return -1;
	}
}

